@@ -20,7 +20,8 @@ enum ProgListType {
     plMovies,
     plCategory,
     plChannel,
-    plTime
+    plTime,
+    plRecordid
 };
 
 
@@ -47,6 +48,7 @@ class ProgLister : public MythDialog
     void setViewFromTime(void);
     void select(void);
     void edit(void);
+    void customEdit(void);
     void upcoming(void);
     void details(void);
     void chooseView(void);
@@ -83,7 +85,7 @@ class ProgLister : public MythDialog
     ProgramList schedList;
 
     QStringList typeList;
-    QStringList categoryList;
+    QStringList genreList;
     QStringList stationList;
 
     XMLParse *theme;
@@ -130,11 +132,11 @@ class ProgLister : public MythDialog
     MythRemoteLineEdit *powerSubtitleEdit;
     MythRemoteLineEdit *powerDescEdit;
     MythComboBox *powerCatType;
-    MythComboBox *powerCategory;
+    MythComboBox *powerGenre;
     MythComboBox *powerStation;
     MythPushButton *powerOkButton;
 
-    void powerStringToSQL(const QString &qphrase, QString &output, 
+    bool powerStringToSQL(const QString &qphrase, QString &output, 
                           MSqlBindings &bindings);
 };
 
