@@ -2,7 +2,6 @@
 #define MAINSERVER_H_
 
 #include <qmap.h>
-#include <qsocket.h>
 #include <qtimer.h>
 #include <qurl.h>
 #include <qmutex.h>
@@ -18,11 +17,11 @@ using namespace std;
 #include "scheduler.h"
 #include "livetvchain.h"
 #include "autoexpire.h"
+#include "mythsocket.h"
 
-class HttpStatus;
 class ProcessRequestThread;
 
-class MainServer : public QObject
+class MainServer : public QObject, public MythSocketCBs
 {
     Q_OBJECT
   public:
@@ -34,26 +33,26 @@ class MainServer : public QObject
 
     void customEvent(QCustomEvent *e);
 
-    void FillProgramInfo(QDomDocument *pDoc, QDomElement &e, ProgramInfo *pInfo);
-    void FillStatusXML(QDomDocument *pDoc);
-
     bool isClientConnected();
     void ShutSlaveBackendsDown(QString &haltcmd);
 
-    void ProcessRequest(RefSocket *sock); 
+    void ProcessRequest(MythSocket *sock); 
     void MarkUnused(ProcessRequestThread *prt);
 
+    void readyRead(MythSocket *socket);
+    void connectionClosed(MythSocket *socket);
+    void connectionFailed(MythSocket *socket) { (void)socket; }
+    void connected(MythSocket *socket) { (void)socket; }
+
     void DeletePBS(PlaybackSock *pbs);
 
   protected slots:
     void reconnectTimeout(void);
-    void masterServerDied(void);
     void deferredDeleteSlot(void);
+    void autoexpireUpdate(void);
 
   private slots:
-    void newConnection(RefSocket *);
-    void endConnection(RefSocket *);
-    void readSocket();
+    void newConnection(MythSocket *);
 
   private:
     typedef struct deletestruct
@@ -67,10 +66,10 @@ class MainServer : public QObject
         bool forceMetadataDelete;
     } DeleteStruct;
 
-    void ProcessRequestWork(RefSocket *sock);
+    void ProcessRequestWork(MythSocket *sock);
     void HandleAnnounce(QStringList &slist, QStringList commands, 
-                        RefSocket *socket);
-    void HandleDone(QSocket *socket);
+                        MythSocket *socket);
+    void HandleDone(MythSocket *socket);
 
     void HandleIsActiveBackendQuery(QStringList &slist, PlaybackSock *pbs);
     void HandleQueryRecordings(QString type, PlaybackSock *pbs);
@@ -106,6 +105,7 @@ class MainServer : public QObject
     void HandleIsRecording(QStringList &slist, PlaybackSock *pbs);
     void HandleCheckRecordingActive(QStringList &slist, PlaybackSock *pbs);
     void HandleFillProgramInfo(QStringList &slist, PlaybackSock *pbs);
+    void HandleSetChannelInfo(QStringList &slist, PlaybackSock *pbs);
     void HandleRemoteEncoder(QStringList &slist, QStringList &commands,
                              PlaybackSock *pbs);
     void HandleLockTuner(PlaybackSock *pbs);
@@ -121,33 +121,39 @@ class MainServer : public QObject
     void HandleSetBookmark(QStringList &tokens, PlaybackSock *pbs);
     void HandleSettingQuery(QStringList &tokens, PlaybackSock *pbs);
     void HandleSetSetting(QStringList &tokens, PlaybackSock *pbs);
-    void HandleVersion(QSocket *socket, QString version);
-    void HandleBackendRefresh(QSocket *socket);
+    void HandleVersion(MythSocket *socket, QString version);
+    void HandleBackendRefresh(MythSocket *socket);
     void HandleQueryLoad(PlaybackSock *pbs);
     void HandleQueryUptime(PlaybackSock *pbs);
     void HandleQueryMemStats(PlaybackSock *pbs);
     void HandleBlockShutdown(bool blockShutdown, PlaybackSock *pbs);
     
-    void SendResponse(QSocket *pbs, QStringList &commands);
+    void SendResponse(MythSocket *pbs, QStringList &commands);
 
     void getGuideDataThrough(QDateTime &GuideDataThrough);
 
     PlaybackSock *getSlaveByHostname(QString &hostname);
-    PlaybackSock *getPlaybackBySock(QSocket *socket);
+    PlaybackSock *getPlaybackBySock(MythSocket *socket);
     FileTransfer *getFileTransferByID(int id);
-    FileTransfer *getFileTransferBySock(QSocket *socket);
+    FileTransfer *getFileTransferBySock(MythSocket *socket);
 
     QString LocalFilePath(QUrl &url);
 
     static void *SpawnDeleteThread(void *param);
-    void DoDeleteThread(DeleteStruct *ds);
+    void DoDeleteThread(const DeleteStruct *ds);
+    void DoDeleteInDB(const DeleteStruct *ds, const ProgramInfo *pginfo);
 
     LiveTVChain *GetExistingChain(QString id);
-    LiveTVChain *GetExistingChain(QSocket *sock);
+    LiveTVChain *GetExistingChain(MythSocket *sock);
     LiveTVChain *GetChainWithRecording(ProgramInfo *pginfo);
     void AddToChains(LiveTVChain *chain);
     void DeleteChain(LiveTVChain *chain);
 
+    static int  DeleteFile(const QString &filename, bool followLinks);
+    static int  OpenAndUnlink(const QString &filename);
+    static bool TruncateAndClose(const AutoExpire *expirer,
+                                 int fd, const QString &filename);
+
     QPtrList<LiveTVChain> liveTVChains;
     QMutex liveTVChainsLock;
 
@@ -160,8 +166,6 @@ class MainServer : public QObject
 
     QString recordfileprefix;
 
-    HttpStatus *statusserver;
-
     QTimer *masterServerReconnect;
     PlaybackSock *masterServer;
     
@@ -187,6 +191,10 @@ class MainServer : public QObject
     QMutex deferredDeleteLock;
     QTimer *deferredDeleteTimer;
     QValueList<DeferredDeleteStruct> deferredDeleteList;
+
+    QTimer *autoexpireUpdateTimer;
+
+    static QMutex truncate_and_close_lock;
 };
 
 #endif
